
'''
This module implements wrapper functions to access and modify easily the STED
parameters through specpy.
'''

import time
import numpy
import pickle

try:
    import specpy
    # from specpy import Imspector
    # im = Imspector()
    im = specpy.get_application()
    measurement = im.active_measurement()
except (ModuleNotFoundError, RuntimeError) as err:
    print(err)
    print("Calling these functions might raise an error.")
    print("\nFalling back to the debug files...")

    from .debug import Imspector
    im = Imspector()
    measurement = im.active_measurement()

def get_config(message=None, image=None):
    '''Fetch and return the active configuration in Imspector.

    :param message: If defined, print the following message.

    :returns: The active configuration (specpy Configuration object).
    '''
    if message is not None:
        print(message)
    print("Manually select imaging configuration then press enter.")
    input()
    return measurement.active_configuration()

def acquire_multi_saveasmsr(configs, savepath):
    '''Activate the given configuration and acquire an image stack.

    :param conf: A configuration object.
    :param savepath: a path to a file

    :return: An image stack (3d array) and the acquisition time (seconds).
    '''
    for conf in configs:
        measurement.activate(conf)
        start = time.time()
        im.run(measurement)
        end = time.time()
        print(end-start)
        stacks = [conf.stack(i) for i in range(conf.number_of_stacks())]
        x, y = get_offsets(conf)
        print("Acquiring with configuration", conf.name(), "at offset x:", x, ", y:", y)
    measurement.save_as(savepath, True)
    # conf.stack(conf.name())
    # chop the first 2 lines because of imaging problems I guess
    # chop 0.08 seconds because life
    return [[image[2:].copy() for image in stack.data()[0]] for stack in stacks], end - start - 0.08

def acquire_multi(configs):
    '''Activate the given configuration and acquire an image stack.

    :param conf: A configuration object.
    :param savepath: a path to a file

    :return: An image stack (3d array).
    '''
    finalstack = []
    for conf in configs:
        measurement.activate(conf)
        start = time.time()
        im.run(measurement)
        end = time.time()
        stacks = [conf.stack(i) for i in range(conf.number_of_stacks())]
        finalstack.append([[image[2:].copy() for image in stack.data()[0]] for stack in stacks])
        x, y = get_offsets(conf)
        print("Acquiring with configuration", conf.name(), "at offset x:", x, ", y:", y)
    # conf.stack(conf.name())
    # chop the first 2 lines because of imaging problems I guess
    # chop 0.08 seconds because life
    return finalstack

def clone(conf):
    '''Clones the corresponding configuration in the measurement, activates and returns the clone.

    :param conf: A configuration object.
    :param name: Name of the clone measurement

    :returns: A configuration object
    '''
    return measurement.clone(conf)

def get_num_channels(conf):
    '''Fetch and return the number of channels of a configuration object

    :param conf: A configuration object

    :returns : A number of channels
    '''
    return len(conf.parameters("ExpControl/measurement/channels"))

def get_params(conf):
    '''Fetch and return the parameters of a configuration object.

    :param conf: A configuration object.

    :returns: A dict of parameters.
    '''
    return conf.parameters("ExpControl")

def get_power(conf, laser_id):
    '''Fetch and return the power of a laser in a specific configuration.

    :param conf: A configuration object.
    :param laser_id: ID of the laser in Imspector (starting from 0).

    :returns: The power (%).
    '''
    params = conf.parameters("ExpControl/measurement")
    #TODO: should we return a ratio instead?
    return params["lasers"][laser_id]["power"]["calibrated"]

def get_pixelsize(conf):
    '''Fetch and return the pixel size in a specific configuration.

    :param conf: A configuration object.

    :returns: Tuple of (x, y) pixel sizes (m).
    '''
    x = conf.parameters("ExpControl/scan/range/x/psz")
    y = conf.parameters("ExpControl/scan/range/y/psz")
    return x, y

def get_resolution(conf):
    '''Fetch and return the resolution in a specific configuration.

    :param conf: A configuration object.

    :returns: Tuple of (x, y) resolutions (m).
    '''
    x = conf.parameters("ExpControl/scan/range/x/res")
    y = conf.parameters("ExpControl/scan/range/y/res")
    return x, y

def get_imagesize(conf):
    '''Fetch and return the image size in a specific configuration.

    :param conf: A configuration object.

    :returns: Tuple of (x, y) image sizes (m).
    '''
    x = conf.parameters("ExpControl/scan/range/x/len")
    y = conf.parameters("ExpControl/scan/range/y/len")
    return x, y

def get_offsets(conf):
    '''Fetch and return the offsets in a specific configuration.

    :param conf: A configuration object.

    :returns: Tuple of (x, y) offsets.
    '''
    x = conf.parameters("ExpControl/scan/range/x/off")
    y = conf.parameters("ExpControl/scan/range/y/off")
    return x, y

def get_coarse_range(conf):
    '''Fetch and return the coarse range in a specific configuration.

    :param conf: A configuration object.

    :returns: Tuple of (x, y, z) offsets.
    '''
    x = conf.parameters("ExpControl/scan/range/coarse_x/g_off")
    y = conf.parameters("ExpControl/scan/range/coarse_y/g_off")
    z = conf.parameters("ExpControl/scan/range/coarse_z/g_off")
    return x, y, z

def get_dwelltime(conf,channel_id=0):
    ''' Fetch and return the pixel dwell time in a specific configuration.

    :param conf: A configuration object.

    :returns: The dwell time (s).
    '''
    params = conf.parameters("ExpControl/measurement/pixel_steps")
    return params['step_duration'][channel_id]

def get_overview(conf, prefix="Overview ", name=None):
    """Fetch and return the overview of a specific configuration.
    
    :param conf: A configuration object.
    :param prefix: Prefix of the overview.
    :param name: Name of the overview.

    :return: The overview image.
    """
    if name is None:
        print(prefix)
        print("Type the name of the overview then press enter.")
        name = str(input())
        overview = prefix + name
    else:
        overview = prefix + name
    # print('overview')

    # # There could be something similar to
    for stack_name in conf.stack_names():
        if name in stack_name:
            break
    return conf.stack(stack_name).data()[0][0]

    # return conf.stack(overview).data()[0][0]

def get_image(conf, name):
    """Fetch and return the image of a specific configuration.
    
    :param conf: A configuration object.
    :param name: Name of the image.

    :return: The image.
    """
    return conf.stack(name).data()[0][0]

def get_spectral_min(conf, channel_id=0):
    ''' Fetch and return the minimal spectral range in a specific configuration.

    :param conf: A configuration object.
    :param channel_id: ID of the channel

    :returns: The minimal spectral range
    '''
    return conf.parameters(f"ExpControl/measurement/channels/{channel_id}/detsel/spectral_range/min")

def get_spectral_max(conf, channel_id=0):
    ''' Fetch and return the maximal spectral range in a specific configuration.

    :param conf: A configuration object.
    :param channel_id: ID of the channel

    :returns: The maximal spectral range
    '''
    return conf.parameters(f"ExpControl/measurement/channels/{channel_id}/detsel/spectral_range/max")

def set_pixelsize(conf, x, y):
    '''Sets the pixel size

    :param conf: Configuration window
    :param x: pixel size in x
    :param y: pixel size in y
    '''
    conf.set_parameters("ExpControl/scan/range/x/psz", x)
    conf.set_parameters("ExpControl/scan/range/y/psz", y)

def set_offsets(conf, x, y):
    '''Set the offsets in a specific configuration.

    :param conf: A configuration object.
    :param x: The x offset.
    :param y: The y offset.
    '''
    conf.set_parameters("ExpControl/scan/range/x/off", x)
    conf.set_parameters("ExpControl/scan/range/y/off", y)

def set_coarse_range(conf, x, y, z):
    '''Set the coarse range in a specific configuration.

    :param conf: A configuration object.

    :returns: Tuple of (x, y, z) offsets.
    '''
    conf.set_parameters("ExpControl/scan/range/coarse_x/g_off", x)
    conf.set_parameters("ExpControl/scan/range/coarse_y/g_off", y)
    conf.set_parameters("ExpControl/scan/range/coarse_z/g_off", z)
    return x, y, z

def set_imagesize(conf, width, height):
    '''Set the imagesize in a specific configuration.

    :param conf: A configuration object.
    :param width: width (m)
    :param height: height (m)
    '''
    # print("The function receives {}, which is of type {}... It should be of type {}".format(resolution, type(resolution[0]), type(get_resolution(conf)[0])))
    conf.set_parameters("ExpControl/scan/range/x/len", width)
    conf.set_parameters("ExpControl/scan/range/y/len", height)
    # print("The resolution of the window is now : {}. It should be {} {}".format(get_imagesize(conf), width, height))

def set_numberframe(conf, num):
    '''Set the number of frame in a xyt configuration.

    :param conf: A configuration object.
    :param num: The number of frame.
    '''
    conf.set_parameters("ExpControl/scan/range/t/res", num)

def set_power(conf, power, laser_id,channel_id=0):
    '''Set the power of a laser in a specific configuration.

    :param conf: A configuration object.
    :param laser_id: ID of the laser in Imspector (starting from 0).
    :param power: Power of the laser in [0, 1].
    '''
    params = conf.parameters("ExpControl/measurement")

    if laser_id == 0:
        print('405 nm')
        params["channels"][channel_id]["lasers"][laser_id]["power"]["calibrated"] = power*1e-3
    else:
        params["channels"][channel_id]["lasers"][laser_id]["power"]["calibrated"] = power
    conf.set_parameters("ExpControl/measurement/channels", params["channels"])

def set_scan_axes(conf, axes):
    '''Set the scan axes of the configuration.

    :param conf: A configuration object.
    :param axes: A `str` of the active axes
    '''
    scan_axes = []
    for ax in axes:
        scan_axes.append("ExpControl {}".format(ax.upper()))
    scan_axes = scan_axes + ["None"] * (4 - len(axes))
    conf.set_parameters("Measurement/axes/scan_axes", scan_axes)

def set_dwelltime(conf, dwelltime, channel_id=0):
    '''Set the pixel dwell time in a specific configuration.

    :param conf: A configuration object.
    :param dwelltime: Pixel dwell time (s).
    '''
    params = conf.parameters("ExpControl/measurement/pixel_steps")
    params['step_duration'][channel_id]=dwelltime
    conf.set_parameters("ExpControl/measurement/pixel_steps", params)

def activate_linestep(conf, status=True):
    """Activate the line step in a specific configuration.
    
    :param conf: A configuration object.
    :param status: A `bool` to activate or deactivate the line step.
    """
    params = conf.parameters("ExpControl/measurement")
    params["line_steps"]["active"]=status
    conf.set_parameters("ExpControl/measurement", params)

def activate_pixelstep(conf, status=True):
    """Activate the pixel step in a specific configuration.
    
    :param conf: A configuration object.
    :param status: A `bool` to activate or deactivate the pixel step.
    """
    params = conf.parameters("ExpControl/measurement")
    params["pixel_steps"]["active"]=status
    conf.set_parameters("ExpControl/measurement", params)

def set_linestep(conf, linestep, step_id):
    '''Set the line step of a specific channel in a specific configuration.

    :param conf: A configuration object.
    :param linestep: Line step.
    :param step_id: ID of the line step in Imspector (starting from 0).
    '''
    step_values = conf.parameters("ExpControl/measurement/line_steps/step_duration")
    step_values[step_id] = int(linestep)
    conf.set_parameters("ExpControl/measurement/line_steps/step_duration", step_values)


def set_frametrig(conf,state):
    ''' Set the state of the frame trigger,

    :param conf: A configuration object
    :param state: Boolean
    '''
    trigger = conf.parameters("ExpControl/trigger")
    trigger["frametrig_use"] = state
    conf.set_parameters("ExpControl/trigger", trigger)


def set_chans_on(conf, line_id, step_id):
    """Set the channels on in a specific configuration.
    
    :param conf: A configuration object
    :param line_id: ID of the line in Imspector (starting from 0)
    :param step_id: ID of the step in Imspector (starting from 0)
    """
    chans_on = conf.parameters("ExpControl/gating/linesteps/chans_on")
    chans_on[line_id][step_id] = True
    conf.set_parameters("ExpControl/gating/linesteps/chans_on", chans_on)

def set_3d(conf, laser_id, power_dist):
    '''Activates the 3d phase modulation and sets the power dist

    :param conf: A configuration object
    :param laser_id: ID of the laser in Imspector (starting from 0)
    :param power_dist: A `float` of the power distribution. (0 is 2D, 100 is 3D)
    '''
    # ensures 3d phase modulation is active
    conf.set_parameters("ExpControl/measurement/allow_custom_phase_patterns", True)
    # Changes the 3d power dist
    lasers = conf.parameters("ExpControl/measurement/lasers")
    lasers[laser_id]["three_d"]["power_dist"] = power_dist
    conf.set_parameters("ExpControl/measurement/lasers", lasers)

def set_defocus(conf, SLM_id, value):
    '''Set the defocus value of the SLM

    :param conf: A configuration object
    :param SLM_id: ID of the SLM (a `str` in {SLM_775, SLM_595})
    :param value: A `float` of the defocus value
    '''
    conf.set_parameters(f"{SLM_id}/pattern_offsets/defocus", value)

def set_spherical(conf, SLM_id, value):
    '''Set the sperical value of the SLM

    :param conf: A configuration object
    :param SLM_id: ID of the SLM (a `str` in {SLM_775, SLM_595})
    :param value: A `float` of the spherical value
    '''
    conf.set_parameters(f"{SLM_id}/pattern_offsets/spherical_aberation_1", value)

def set_spectral_min(conf, value, channel_id=0):
    ''' Sets the minimal spectral range in a specific configuration.

    :param conf: A configuration object.
    :param channel_id: ID of the channel

    :returns: The minimal spectral range
    '''
    conf.set_parameters(f"ExpControl/measurement/channels/{channel_id}/detsel/spectral_range/min", value)

def set_spectral_max(conf, value, channel_id=0):
    ''' Sets the maximal spectral range in a specific configuration.

    :param conf: A configuration object.
    :param channel_id: ID of the channel

    :returns: The maximal spectral range
    '''
    conf.set_parameters(f"ExpControl/measurement/channels/{channel_id}/detsel/spectral_range/max", value)

####  RESCue Parameters   ###

def set_rescue_signal_level(conf, signal_level, channel_id):
    '''Set the RESCue signal level in a specific configuration.

    :param conf: A configuration object.
    :param signal_level: Signal level of RESCue.
    :param channel_id: ID of the RESCue channel in Imspector (starting from 0).
    '''
    channels = conf.parameters("ExpControl/measurement/channels")
    channels[channel_id]["rescue"]["signal_level"] = signal_level
    conf.set_parameters("ExpControl/measurement/channels", channels)


def set_rescue_strength(conf, strength, channel_id):
    '''Set the RESCue strength in a specific configuration.

    :param conf: A configuration object.
    :param strength: Strength of RESCue.
    :param channel_id: ID of the RESCue channel in Imspector (starting from 0).
    '''
    channels = conf.parameters("ExpControl/measurement/channels")
    channels[channel_id]["rescue"]["strength"] = strength
    conf.set_parameters("ExpControl/measurement/channels", channels)

def set_LTh_auto(conf, value, channel_id):
    """Set the lower threshold automatic mode

    :param conf: A configuration object
    :param value: A `bool` wheter the lower threshold use automatic values
    :param channel_id: ID of the channel
    """
    channels = conf.parameters("ExpControl/measurement/channels")
    channels[channel_id]["rescue"]["LTh_auto"] = value
    conf.set_parameters("ExpControl/measurement/channels", channels)

def set_LTh_manual(conf, channel_id):
    """Set the lower threshold manual mode
    
    :param conf: A configuration object
    :param channel_id: ID of the channel
    """
    channels = conf.parameters("ExpControl/rescue/channels")
    channels[channel_id]["set_thresholds_manually"] = True
    conf.set_parameters("ExpControl/rescue/channels", channels)

def set_LTh_num_times(conf, num_times, channel_id):
    """Set number of lower threshold values

    :param conf: A configuration object
    :param num_times: An `int` of the number of threshold values
    :param channel_id: ID of the channel (Starting from 0)
    """
    channels = conf.parameters("ExpControl/measurement/channels")
    channels[channel_id]["rescue"]["LTh_num_times"] = num_times
    conf.set_parameters("ExpControl/measurement/channels", channels)

def set_LTh_thresholds(conf, thresholds, channel_id):
    """Set lower threshold values

    :param conf: A configuration object
    :param thresholds: A `list` of threshold values
    :param channel_id: ID of the channel (Starting from 0)
    """
    if not isinstance(thresholds, (list, tuple)):
        thresholds = [thresholds]
    if len(thresholds) != 4:
        thresholds = thresholds + [0] * (4 - len(thresholds))
    channels = conf.parameters("ExpControl/measurement/channels")
    channels[channel_id]["rescue"]["LTh_thresholds"] = list(map(int, thresholds))
    conf.set_parameters("ExpControl/measurement/channels", channels)

def set_LTh_times(conf, times, channel_id):
    """Set lower threshold time values

    :param conf: A configuration object
    :param thresholds: A `list` of times
    :param channel_id: ID of the channel (Starting from 0)
    """
    if not isinstance(times, (list, tuple)):
        times = [times]
    if len(times) != 4:
        times = times + [0] * (4 - len(times))
    channels = conf.parameters("ExpControl/measurement/channels")
    channels[channel_id]["rescue"]["LTh_times"] = times
    conf.set_parameters("ExpControl/measurement/channels", channels)

def set_uth_manual(conf, channel_id):
    """Set the upper threshold manual mode
    
    :param conf: A configuration object
    :param channel_id: ID of the channel
    """
    channels = conf.parameters("ExpControl/rescue/channels")
    channels[channel_id]["UTh_manual"] = True
    channels[channel_id]["UTh_use"] = True

    conf.set_parameters("ExpControl/rescue/channels", channels)

def set_UTh_threshold(conf, threshold, channel_id):
    """Set upper threshold value

    :param conf: A configuration object
    :param threshold: A threshold value
    :param channel_id: ID of the channel (Starting from 0)
    """
    channels = conf.parameters("ExpControl/measurement/channels")
    channels[channel_id]["rescue"]["UTh_threshold"] = int(threshold)
    conf.set_parameters("ExpControl/measurement/channels", channels)

def set_UTh_auto(conf, value, channel_id):
    """Set the upper threshold automatic mode

    :param conf: A configuration object
    :param value: A `bool` wheter the lower threshold use automatic values
    :param channel_id: ID of the channel
    """
    channels = conf.parameters("ExpControl/measurement/channels")
    channels[channel_id]["rescue"]["UTh_auto"] = value
    conf.set_parameters("ExpControl/measurement/channels", channels)

def turn_uth_off1(conf, channel_id):
    """Turn off the upper threshold
    
    :param conf: A configuration object
    :param channel_id: ID of the channel
    """
    channels = conf.parameters("ExpControl/rescue/channels")
    channels[channel_id]["UTh_manual"] = True
    #channels[channel_id]["UTh_use"] = False
    conf.set_parameters("ExpControl/rescue/channels", channels)

def turn_uth_off2(conf, channel_id):
    """Turn off the upper threshold
    
    :param conf: A configuration object
    :param channel_id: ID of the channel
    """
    channels = conf.parameters("ExpControl/rescue/channels")
    #channels[channel_id]["UTh_manual"] = True
    channels[channel_id]["UTh_use"] = False
    conf.set_parameters("ExpControl/rescue/channels", channels)

def turn_on_rescue(conf, channel_id):
    """Turn on the rescue mode
    
    :param conf: A configuration object
    :param channel_id: ID of the channel
    """
    channels = conf.parameters("ExpControl/rescue/channels")
    channels[channel_id]["on"] = True
    conf.set_parameters("ExpControl/rescue/channels", channels)


def turn_off_rescue(conf, channel_id):
    """Turn off the rescue mode
    
    :param conf: A configuration object
    :param channel_id: ID of the channel
    """
    channels = conf.parameters("ExpControl/rescue/channels")
    channels[channel_id]["on"] = False
    channels[channel_id]["rescue_allowed"] = False
    conf.set_parameters("ExpControl/rescue/channels", channels)

def set_auto_blanking(conf, channel_id):
    """Set the auto blanking mode
    
    :param conf: A configuration object
    :param channel_id: ID of the channel
    """
    channels = conf.parameters("ExpControl/rescue/channels")
    channels[channel_id]["auto_blank"] = True
    conf.set_parameters("ExpControl/rescue/channels", channels)

def set_manual_blanking(conf,channel_id):
    """Set the manual blanking mode
    
    :param conf: A configuration object
    :param channel_id: ID of the channel
    """
    channels = conf.parameters("ExpControl/rescue/channels")
    channels[channel_id]["auto_blank"] = False
    conf.set_parameters("ExpControl/rescue/channels", channels)

def set_manual_blanking_lasers(conf,lasers,channel_id):
    """Set the manual blanking mode

    :param conf: A configuration object
    :param lasers: A list of lasers to blank
    :param channel_id: ID of the channel
    """
    blanklasers=[False,False,False,False,False,False,False,False]
    for laser in lasers:
        blanklasers[laser]=True
    channels = conf.parameters("ExpControl/rescue/channels")
    channels[channel_id]["blanking"] = blanklasers
    conf.set_parameters("ExpControl/rescue/channels", channels)

def turn_probe_on(conf, channel_id):
    """Turn on the probe mode
    
    :param conf: A configuration object
    :param channel_id: ID of the channel
    """
    channels = conf.parameters("ExpControl/rescue/channels")
    channels[channel_id]['use_as_probe'] = True
    conf.set_parameters("ExpControl/rescue/channels", channels)

def turn_probe_off(conf, channel_id):
    """Turn off the probe mode
    
    :param conf: A configuration object
    :param channel_id: ID of the channel
    """
    channels = conf.parameters("ExpControl/rescue/channels")
    channels[channel_id]['use_as_probe'] = False
    conf.set_parameters("ExpControl/rescue/channels", channels)

def set_rescue_mode(conf, mode, channel_id):
    """Set the rescue mode

    :param conf: A configuration object
    :param mode: Probe mode to use (Starting at 0)
    :param channel_id: ID of the channel to modify (Starting at 0)
    """
    channels = conf.parameters("ExpControl/measurement/channels")
    channels[channel_id]["rescue"]["rescue_mode"] = mode
    conf.set_parameters("ExpControl/measurement/channels", channels)
###   End of RESCue parameters   ####


def acquire(conf):
    '''Activate the given configuration and acquire an image stack.

    :param conf: A configuration object.

    :return: An image stack (3d array) and the acquisition time (seconds).
    '''
    measurement.activate(conf)
    start = time.time()
    im.run(measurement)
    end = time.time()
    stacks = [conf.stack(i) for i in range(conf.number_of_stacks())]
    x, y = get_offsets(conf)
    print("Acquiring with configuration", conf.name(), "at offset x:", x, ", y:", y)

    #conf.stack(conf.name())
    # chop the first 2 lines because of imaging problems I guess
    # chop 0.08 seconds because life
    return [[image.copy() for image in stack.data()[0]] for stack in stacks], end - start - 0.08

def acquire_saveasmsr(conf,savepath):
    '''Activate the given configuration and acquire an image stack.

    :param conf: A configuration object.
    :param savepath: a path to a file

    :return: An image stack (3d array) and the acquisition time (seconds).
    '''
    measurement.activate(conf)
    start = time.time()
    im.run(measurement)
    end = time.time()
    stacks = [conf.stack(i) for i in range(conf.number_of_stacks())]
    x, y = get_offsets(conf)
    print("Acquiring with configuration", conf.name(), "at offset x:", x, ", y:", y)
    measurement.save_as(savepath,True)
    #conf.stack(conf.name())
    # chop the first 2 lines because of imaging problems I guess
    # chop 0.08 seconds because life
    return [[image.copy() for image in stack.data()[0]] for stack in stacks],end - start - 0.08

if __name__ == "__main__":
    import pickle

    conf = get_config()
    params = get_params(conf)
    print(params["measurement"]["channels"][0]["rescue"])
    # with open("test", "wb") as f:
        # pickle.dump(params, f)
    # with open("test", "rb") as f:
        # params = pickle.load(f)
    # set_params(params)



import sys

from matplotlib import pyplot

import numpy

from scipy.optimize import curve_fit
from scipy.spatial.distance import cdist

from skimage import draw, filters


def avg_area(img, radius, point):
    '''Compute the average of the area around the given point.

    :param img: An image (2d array).
    :param radius: Vertical and horizontal radius to consider around point.
    :param point: Coordinates tuple.

    :returns: The average of the area around the given point.
    '''
    x, y = int(point[0]), int(point[1])
    x_max, y_max = img.shape[1] - 1, img.shape[0] - 1
    assert y >= 0 and x >= 0 and y <= y_max and x <= x_max

    y_start, x_start = max(0, y-radius), max(0, x-radius)
    y_end, x_end = min(y_max, y_start+2*radius), min(x_max, x_start+2*radius)
    return numpy.mean(img[y_start:y_end+1, x_start:x_end+1])


def avg_signal(signal, radius, points):
    '''Compute the estimated signal using the average over areas around all the
    given points.

    :param signal: An image (2d array).
    :param radius: Vertical and horizontal radius to consider around points.
    :param points: List of coordinates tuples.

    :returns: The estimated signal.
    '''
    avg_areas = [avg_area(signal, radius, point) for point in points]
    return numpy.mean(avg_areas)


def gaussian_fit(img, start, end):
    values = []
    for delta in [-1, 0, 1]:
        cc, rr = draw.line(*start, *end)
        rr += delta
        positions = numpy.sqrt((rr - start[0])**2 + (cc - start[1])**2)
        values.append(img[rr, cc])
    values = numpy.asarray(values)
    avg_values = numpy.mean(values, axis=0)
    y0 = numpy.mean(values)
    mu = numpy.mean(positions)
    sigma = numpy.std(positions)
    a = numpy.max(avg_values) * 2 * sigma
    try:
        popt = curve_fit(gauss2, positions, avg_values, p0=[y0, a, mu, sigma])[0]
    except (RuntimeError, TypeError, NameError) as err:
        print("Gaussian fit failed")
        print(err)
        pyplot.figure("Failed to fit these data")
        pyplot.plot(positions, avg_values, "bo")
        pyplot.show(block=True)
        popt = None
    return popt, positions, avg_values


def points2regions(points, pixelsize, resolution):
    '''Return the regions corresponding to the selected points.
    '''
    x_pixels, y_pixels = resolution
    x_size, y_size = pixelsize
    print("points2regions")
    print("resolution", x_pixels, "(x),", y_pixels, "(y)")
    print("pixelsize", x_size, "(x),", y_size, "(y)")
    return [((x-x_pixels/2)*x_size, (y-y_pixels/2)*y_size) for (x, y) in points]


def rect2regions(rectangles, pixelsize):
    ''' Outputs the new window size from the selected rectangles
    '''
    x_size, y_size = pixelsize
    new_window_size = []

    for (start_point, end_point) in rectangles:
        if end_point[0] > start_point[0] and end_point[1] > start_point[1]:
            new_window_size.append(((end_point[0]-start_point[0])*x_size, (end_point[1]-start_point[1])*y_size))
        elif end_point[0] < start_point[0] and end_point[1] > start_point[1]:
            new_window_size.append(((start_point[0]-end_point[0])*x_size, (end_point[1]-start_point[1])*y_size))
        elif end_point[0] < start_point[0] and end_point[1] < start_point[1]:
            new_window_size.append(((start_point[0]-end_point[0])*x_size, (start_point[1]-end_point[1])*y_size))
        else:
            new_window_size.append(((end_point[0]-start_point[0])*x_size, (start_point[1]-end_point[1])*y_size))
    return new_window_size
    # return [((end_point[0]-start_point[0])*x_size, (end_point[1]-start_point[1])*y_size) for (start_point, end_point) in rectangles]


def get_rect_center(rectangles, pixelsize, resolution):
    '''Outputs the center of the selected rectangles
    '''
    x_pixels, y_pixels = resolution
    x_size, y_size = pixelsize
    rect_center = []
    print(rectangles)
    for start_point, end_point in rectangles:
        print(start_point)
        if end_point[0] > start_point[0] and end_point[1] > start_point[1]:
            rect_center.append((((start_point[0]+(end_point[0]-start_point[0])/2)-x_pixels/2)*x_size, ((start_point[1]+(end_point[1]-start_point[1])/2)-y_pixels/2)*y_size))
        elif end_point[0] < start_point[0] and end_point[1] > start_point[1]:
            rect_center.append((((end_point[0]+(start_point[0]-end_point[0])/2)-x_pixels/2)*x_size, ((start_point[1]+(end_point[1]-start_point[1])/2)-y_pixels/2)*y_size))
        elif end_point[0] < start_point[0] and end_point[1] < start_point[1]:
            rect_center.append((((end_point[0]+(start_point[0]-end_point[0])/2)-x_pixels/2)*x_size, ((end_point[1]+(start_point[1]-end_point[1])/2)-y_pixels/2)*y_size))
        else:
            rect_center.append((((start_point[0]+(end_point[0]-start_point[0])/2)-x_pixels/2)*x_size, ((end_point[1]+(start_point[1]-end_point[1])/2)-y_pixels/2)*y_size))
    # return [(((start_point[0]+(end_point[0]-start_point[0])/2)-x_pixels/2)*x_size, ((start_point[1]+(end_point[1]-start_point[1])/2)-y_pixels/2)*y_size) for (start_point, end_point) in rectangles]
    return rect_center

def gauss(x, a, mu, sigma):
    return a * numpy.exp(-(x-mu)**2/(2*sigma**2))


def gauss2(x, y0, a, mu, sigma):
    w = 2 * sigma
    return y0 + (a / w * numpy.sqrt(numpy.pi/2)) * numpy.exp(-2*(x-mu)**2/w**2)


def get_closer(last_options, last_preferred, options, metric):
    dists = cdist(options, [last_options[last_preferred]], metric=metric)
    return numpy.argmin(dists)


def get_foreground(img):
    val = filters.threshold_otsu(img)
    return img > val


if __name__ == "__main__":
    import skimage.io
    path = "Test foreground background/35"
    img = skimage.io.imread(path+".tiff")
    fg = get_foreground(img)
    img_fg = numpy.zeros_like(img)
    img_fg[fg] = 255
    skimage.io.imsave(path+"_fg.pdf", img_fg)




'''This module contains the functions/dialogues to interact with users.
Ex. Ask user to pick background/foreground points.
'''

import numpy
import random

from matplotlib import pyplot, widgets, patches, rcParams
from matplotlib.widgets import Button
from mpl_toolkits.axes_grid1 import make_axes_locatable
#pyplot.ion()

try:
    from . import microscope
except:
    print("Could not load microscope interface. Some functions may not be available.")
from . import utils

class LinePicker:
    def __init__(self, figure):
        figure.canvas.mpl_connect("button_press_event", self.on_press)
        figure.canvas.mpl_connect("button_release_event", self.on_release)
        self.pressed = False
        self.start = []
        self.end = []

    def on_press(self, event):
        self.start.append((int(event.xdata), int(event.ydata)))
        self.pressed = True

    def on_release(self, event):
        if self.pressed:
            point = (int(event.xdata), int(event.ydata))
            if point == self.start[-1]:
                self.start.pop(-1)
                print("Pick lines, not points!")
            else:
                self.end.append(point)
            self.pressed = False

    def get_lines(self):
        if not self.pressed:
            return self.start, self.end


class SidePicker:
    def __init__(self, figure):
        figure.canvas.mpl_connect("button_press_event", self.on_press)
        figure.canvas.mpl_connect("button_release_event", self.on_release)
        self.pressed = False
        self.start = []
        self.end = []
        self.side = []

    def on_press(self, event):
        self.start.append((int(event.xdata), int(event.ydata)))
        self.side.append((int(event.xdata), int(event.ydata)))
        self.pressed = True

    def on_release(self, event):
        if self.pressed:
            point = (int(event.xdata), int(event.ydata))
            if point == self.start[-1] and self.end != []:
                self.start.pop(-1)
                print("Pick lines, not points!")
            elif point == self.start[-1] and self.end == []:
                self.side.append(point)
            else:
                self.end.append(point)
            self.pressed = False

    def get_lines(self):
        if not self.pressed:
            return self.start, self.end

    def get_point(self):
        if not self.pressed:
            return self.point



class PointPicker:
    def __init__(self, figure):
        figure.canvas.mpl_connect("button_press_event", self.on_press)
        self.points = []

    def on_press(self, event):
        self.points.append((int(event.xdata), int(event.ydata)))


class RectPicker:
    def __init__(self, figure):
        figure.canvas.mpl_connect("button_press_event", self.on_press)
        figure.canvas.mpl_connect("button_release_event", self.on_release)
        figure.canvas.mpl_connect("motion_notify_event", self.on_motion)
        self.pressed = False
        self.start = []
        self.end = []
        self.ax = pyplot.gca()
        self.rect = patches.Rectangle((0,0), 1, 1, fill=False, linewidth=2, linestyle='dashed', edgecolor="#FFFFFF")
        self.ax.add_patch(self.rect)
        self.patches = []

    def on_press(self, event):
        if event.inaxes == self.ax:
            self.start.append((int(event.xdata), int(event.ydata)))
            self.pressed = True

    def on_release(self, event):
        if self.pressed and event.inaxes == self.ax:
            point = (int(event.xdata), int(event.ydata))
            if point == self.start[-1]:
                self.start.pop(-1)
                print("Pick rectangles, not points!")
            else:
                self.end.append(point)
                for p in [ patches.Rectangle((self.start[i][0], self.start[i][1]),
                                              self.end[i][0] - self.start[i][0],
                                              self.end[i][1] - self.start[i][1],
                                              fill = False, linewidth = 2,
                                              linestyle = "dashed", edgecolor = "#FFFFFF")
                                              for i in list(range(0, len(self.start)))
                         ]:
                        self.ax.add_patch(p)
                        self.patches.append(p)
                self.pressed = False
            return self.patches

    def on_motion(self, event):
        if self.pressed and event.inaxes == self.ax:
            # print("x position : {}, y position {}".format(event.xdata, event.ydata))
            #for i in list(range(0, len(self.start))):
            i = len(self.start) - 1
            try:
                self.rect.set_width(event.xdata - self.start[i][0])
                self.rect.set_height(event.ydata - self.start[i][1])
                self.rect.set_xy((self.start[i][0], self.start[i][1]))
                self.ax.figure.canvas.draw()
            except TypeError:
                print("Oops! You went out of the image... Come back in!")

    def update(self):
        self.patches[-1].remove()
        self.ax.figure.canvas.draw()

    def get_rectangle(self):
        if not self.pressed:
            return self.start, self.end

class RectMover:
    def __init__(self, figure, rectangles=None):

        self.fig = figure
        self.ax = pyplot.gca()
        self.selected = None
        self.is_pressed = False

        self.rectangles = [
            patches.Rectangle((x, y), endx - x, endy - y, alpha=1, fill=None, edgecolor="white", linestyle="dashed", linewidth=2)
            for ((x, y), (endx, endy)) in rectangles
        ]
        for rect in self.rectangles:
            self.ax.add_patch(rect)

        self.fig.canvas.mpl_connect('button_press_event', self.on_press)
        self.fig.canvas.mpl_connect('button_release_event', self.on_release)
        self.fig.canvas.mpl_connect('motion_notify_event', self.motion_notify)

    def update_position(self, xy):

        x, y = xy

        w = self.selected.get_width()
        h = self.selected.get_height()
        self.selected.set_xy((x-w/2, y-h/2))

        self.fig.canvas.draw()

    def on_press(self, event):

        xpress, ypress = event.xdata, event.ydata
        if event.inaxes:
            for rect in self.rectangles:
                if rect.contains_point(self.ax.transData.transform((xpress, ypress))):
                    self.is_pressed = True
                    self.selected = rect
                    self.update_position((xpress, ypress))

    def on_release(self, event):
        self.is_pressed = False
        self.selected = None

    def motion_notify(self, event):

        if event.inaxes:
            if self.is_pressed:
                xpress, ypress = event.xdata, event.ydata
                self.update_position((xpress, ypress))

    def get_rectangle(self):
        start, end = [], []
        # List of patches
        for rect in self.ax.patches:
        # for rect in self.rectangles:
            start.append(
                (rect.get_x(), rect.get_y())
            )
            end.append(
                (rect.get_x() + rect.get_width(), rect.get_y() + rect.get_height())
            )
        return start, end

def get_lines(img, n):
    lines = []
    lines_left = n
    while lines_left > 0:
        fig = pyplot.figure(figsize=(20, 20))
        # fig.canvas.set_window_title("Pick at least {} lines".format(lines_left))
        lp = LinePicker(fig)
        # pyplot.imshow(img, interpolation=None, vmin=0, vmax=int(0.9*numpy.max(img)), picker=True)
        pyplot.imshow(img, interpolation=None, picker=True)
        pyplot.colorbar()
        pyplot.grid(True)
        pyplot.show(block=True)
        lines.extend([(s, e) for s, e in zip(*lp.get_lines())])
        lines_left -= len(lines)
        if not lines:
            print("early return in function get_lines (user.py)")
            break
    return lines


def get_side(img, n):
    lines = []
    lines_left = n
    while lines_left > 0:
        fig = pyplot.figure(figsize=(8, 8))
        # fig.canvas.set_window_title("Pick at least {} lines".format(lines_left))
        lp = SidePicker(fig)
        print(lp.get_point())
        # pyplot.imshow(img, interpolation=None, vmin=0, vmax=int(0.9*numpy.max(img)), picker=True)
        pyplot.imshow(img, interpolation=None, picker=True)
        pyplot.colorbar()
        pyplot.grid(True)
        pyplot.show(block=True)
        lines.extend([(s, e) for s, e in zip(*lp.get_lines())])
        lines_left -= len(lines)
        if not lines:
            print("early return in function get_lines (user.py)")
            break
    return lines


def get_points(img, n, label="", rectangles=None):
    points = []
    points_left = n
    while points_left > 0:
        fig = pyplot.figure()
        # fig.canvas.set_window_title("Pick at least {} {}".format(points_left, label))
        if rectangles is not None:
            ax = pyplot.gca()
            for p in [ patches.Rectangle((start[0], start[1]),
                                          end[0] - start[0],
                                          end[1] - start[1],
                                          fill = False, linewidth = 2,
                                          linestyle = "dashed", edgecolor = "#FFFFFF")
                                          for start, end in rectangles
                             ]:
                            ax.add_patch(p)
            string = [str(x) for x in range(len(rectangles))]
            [ax.annotate(string[itt], xy=start, xytext=start, horizontalalignment="right",
                verticalalignment="bottom", color="white") for itt, (start, end) in enumerate(rectangles)]
        pp = PointPicker(fig)
        image = pyplot.imshow(img, interpolation=None, picker=True, cmap="hot")
        image.set_clim([0, 0.3*numpy.amax(img)])
        pyplot.colorbar()
        pyplot.grid(True)
        pyplot.show(block=True)
        points.extend(pp.points)
        points_left -= len(points)

        if not points:
            print("early return in function get_points (user.py)")
            break
    return points


def get_rectangles(img, n, label=""):
    answer = "y"
    while answer == "y":
        rect = []
        rect_left = n
        while rect_left > 0:
            rcParams['toolbar'] = "None"
            fig = pyplot.figure(figsize=(12,12))
            # fig.canvas.set_window_title("Pick at least {} rectangle(s) {}.".format(rect_left, label))
            fig.subplots_adjust(bottom=0.2)
            rc = RectPicker(fig)
            image = pyplot.imshow(img, interpolation=None, picker=True, cmap="hot")
            image.set_clim([0, numpy.quantile(img, 0.995)])
            pyplot.colorbar()
            pyplot.grid(True)

            class button_action():
                def close_window(self, event):
                    pyplot.close()

                def clear_selections(self, event):
                    rc.start.pop()
                    rc.end.pop()
                    rc.update()

            callback = button_action()

            button1 = pyplot.axes([0.25, 0.02, 0.5, 0.1])
            divider = make_axes_locatable(button1)
            button2 = divider.append_axes("right", size="100%", pad=0.1)

            clear_button = Button(button1, "Clear last selection\nOnly one (1) !!")
            close_button = Button(button2, "Done!")

            clear_button.on_clicked(callback.clear_selections)
            close_button.on_clicked(callback.close_window)

            pyplot.show(block=True)
            rect.extend([(s, e) for s, e in zip(*rc.get_rectangle())])

            rect_left -= len(rect)
            if not rect:
                print("early return in function get_rectangle (user.py)")
                break
        answer = input("Would you like to restart your selections? (y) If not press enter.")
    return rect

def get_regions(overview, config_overview, n=1, label=None, rectangles=None, actin=False):
    '''Ask the user to select regions and return their offsets.
    '''
    if actin:
        config = microscope.get_config("\nSelect the STED image :)")
        img = microscope.get_image(config)
        points = get_points(img, n, " subregions within the overview {}".format(label))
    else:
        config = config_overview
        #img = microscope.get_overview(config)
        img = microscope.get_overview(config_overview, name = overview)
        points = get_points(img, n, " subregions within the overview {}".format(label), rectangles=rectangles)
    regions = utils.points2regions(points, microscope.get_pixelsize(config), microscope.get_resolution(config))
    x_offset, y_offset = microscope.get_offsets(config)
    regions_offset = [(x + x_offset, y + y_offset) for (x, y) in regions]
    return regions_offset


def get_rect_regions(n=1,overview=None,config_overview=None):
    '''Ask the user to select rectangle regions and returns their positions
    '''
    if overview is None:
        config = microscope.get_config("Setting configuration for overview")
        img = microscope.get_overview(config)
    else:
        config = config_overview
        img = microscope.get_overview(config_overview, name=overview)
    rectangles = get_rectangles(img, n, " subregions within the overview") # Select rectangles
    regions = utils.rect2regions(rectangles, microscope.get_pixelsize(config)) # New window size
    # points = utils.get_rect_center(rectangles, microscope.get_pixelsize(config))
    points = utils.get_rect_center(rectangles, microscope.get_pixelsize(config), microscope.get_resolution(config))
    x_offset, y_offset = microscope.get_offsets(config)
    rect_region_offset = [(x + x_offset, y + y_offset) for (x, y) in points]
    return rect_region_offset, regions, rectangles # returns the offset and the regions dimensions

def move_rect_regions(n=1,overview=None,config_overview=None, rectangles=None, previous=None):
    '''Ask the user to select rectangle regions and returns their positions
    '''
    if overview is None:
        config = microscope.get_config("Setting configuration for overview")
        img = microscope.get_overview(config)
    else:
        config = config_overview
        img = microscope.get_overview(config_overview, name=overview)

    rectangles = move_rectangles(img, rectangles, previous) # Select rectangles

    regions = utils.rect2regions(rectangles, microscope.get_pixelsize(config)) # New window size
    # points = utils.get_rect_center(rectangles, microscope.get_pixelsize(config))
    points = utils.get_rect_center(rectangles, microscope.get_pixelsize(config), microscope.get_resolution(config))
    x_offset, y_offset = microscope.get_offsets(config)
    rect_region_offset = [(x + x_offset, y + y_offset) for (x, y) in points]
    return rect_region_offset, regions, rectangles # returns the offset and the regions dimensions

def move_rectangles(img, rectangles, previous=None):
    answer = "y"
    while answer == "y":
        rect = []

        if not isinstance(previous, type(None)):
            fig, ax = pyplot.subplots(figsize=(10, 10))
            tmp = [
                patches.Rectangle((x, y), endx - x, endy - y, alpha=1, fill=None, edgecolor="white", linestyle="dashed", linewidth=2)
                for ((x, y), (endx, endy)) in rectangles
            ]
            for r in tmp:
                ax.add_patch(r)
            im = ax.imshow(previous, interpolation=None, picker=True, cmap="hot")
            im.set_clim([0, numpy.quantile(previous, 0.995)])
            pyplot.colorbar(im, ax=ax)
            pyplot.grid(True)
            ax.set_title("Previous acquisition")

        fig = pyplot.figure(figsize=(12,12))
        # fig.canvas.set_window_title("Pick at least {} rectangle(s) {}.".format(rect_left, label))
        fig.subplots_adjust(bottom=0.2)
        rc = RectMover(fig, rectangles)
        image = pyplot.imshow(img, interpolation=None, picker=True, cmap="hot")
        image.set_clim([0, numpy.quantile(img, 0.995)])
        pyplot.colorbar()
        pyplot.grid(True)

        class button_action():
            def close_window(self, event):
                pyplot.close()

            def clear_selections(self, event):
                rc.start.pop()
                rc.end.pop()
                rc.update()

        callback = button_action()

        button1 = pyplot.axes([0.25, 0.02, 0.5, 0.1])
        divider = make_axes_locatable(button1)
        button2 = divider.append_axes("right", size="100%", pad=0.1)

        clear_button = Button(button1, "Clear last selection\nOnly one (1) !!")
        close_button = Button(button2, "Done!")

        clear_button.on_clicked(callback.clear_selections)
        close_button.on_clicked(callback.close_window)

        pyplot.show(block=True)

        rect.extend([(s, e) for s, e in zip(*rc.get_rectangle())])

        answer = input("Would you like to restart your selections? (y) If not press enter.")
    return rect

def select(thetas, objectives, *args, **kwargs):
    '''Return the index of the option selected by a user.
    '''
    print("Asking user to select best option...")

    fig = pyplot.figure(figsize=(10, 10))
    ax = fig.gca()

    ax.set_title("Pick the best option by clicking on the point.")

    # 3 points tolerance
    if len(objectives) > 2:
        sc = ax.scatter(thetas[0], thetas[1], s=100, c=thetas[2], marker="o", alpha=0.5, picker=3, **kwargs)
        pyplot.colorbar(sc, ax=ax)
    else:
        ax.scatter(thetas[0], thetas[1], s=200, marker="o", alpha=0.5, picker=3)
    ax.grid(True)
    ax.set_xlabel(objectives[0].label)
    ax.set_ylabel(objectives[1].label)

    def onpick(event):
        global index
        # handle the situation where several points overlap
        if len(objectives) > 2:
            print("Selected points:", event.ind)
            # objectives are minimized (see objectives.py)
            min_z = numpy.min(thetas[2][event.ind])
            candidates = event.ind[thetas[2][event.ind] == min_z]
        else:
            candidates = event.ind
        print("Picking at random in", candidates)
        index = numpy.random.choice(candidates)

    fig.canvas.mpl_connect("pick_event", onpick)
    pyplot.show(block=False)
    while pyplot.waitforbuttonpress():
        pass
    pyplot.close()
    assert index is not None, "User did not pick any point!"
    return index


def select_double(params1, params2, options1, options2, shared, objectives):
    '''Return the indices of options selected by a user.
    '''
    print("Asking user to select best option...")

    fig, axarr = pyplot.subplots(ncols=2, figsize=(25, 10))

    axarr[0].set_title("Pick the best option by clicking on the point:")

    xthetas = [objectives[0].as_objective(theta[0]) for theta in options1]
    ythetas = [objectives[1].as_objective(theta[1]) for theta in options1]

    # 3 points tolerance
    if len(objectives) > 2:
        zthetas = [objectives[2].as_objective(theta[2]) for theta in options1]
        if objectives[2].bounds is None:
            sc1 = axarr[0].scatter(xthetas, ythetas, s=100, c=zthetas, marker="o", alpha=0.5,
                                  vmin=0, picker=3)
        else:
            sc1 = axarr[0].scatter(xthetas, ythetas, s=100, c=zthetas, marker="o", alpha=0.5,
                                  vmin=objectives[2].bounds[0], vmax=objectives[2].bounds[1], picker=3)
        pyplot.colorbar(sc1, ax=axarr[0])
    else:
        sc1 = axarr[0].scatter(xthetas, ythetas, s=200, marker="o", alpha=0.5, picker=3, c=["r"]*len(options1))
    axarr[0].grid(True)
    axarr[0].set_xlabel(objectives[0].label_objective)
    axarr[0].set_ylabel(objectives[1].label_objective)
    if objectives[0].bounds is not None:
        axarr[0].set_xlim(objectives[0].bounds)
    if objectives[1].bounds is not None:
        axarr[0].set_ylim(objectives[1].bounds)

    xthetas2 = [objectives[0].as_objective(theta[0]) for theta in options2]
    ythetas2 = [objectives[1].as_objective(theta[1]) for theta in options2]

    axarr[1].set_title("Pick the best valid option by clicking on the point:")

    # 3 points tolerance
    sc2 = axarr[1].scatter(xthetas2, ythetas2, s=200, marker="o", alpha=0.5, picker=3, c=["r"]*len(options2))
    axarr[1].grid(True)
    axarr[1].set_xlabel(objectives[0].label_objective)
    axarr[1].set_ylabel(objectives[1].label_objective)
    if objectives[0].bounds is not None:
        axarr[1].set_xlim(objectives[0].bounds)
    if objectives[1].bounds is not None:
        axarr[1].set_ylim(objectives[1].bounds)

    params, options, scs = [params1, params2], [options1, options2], [sc1, sc2]

    global idxi
    global idxj
    global valids
    idxi, idxj, valids = None, None, None

    def onpick(event):
        global idxi
        global idxj
        global valids
        global mainax
        global axi
        global axj

        if valids is None:
            mainax = event.artist.axes
            if mainax == axarr[0]:
                axi, axj = 0, 1
            else:
                axi, axj = 1, 0

        if event.artist.axes == mainax:
            # handle the situation where several points overlap
            if len(objectives) > 2:
                print("Selected points:", event.ind)
                # objectives are minimized (see objectives.py)
                min_z = numpy.min(options[axi][:, 2][event.ind])
                candidates = event.ind[options[axi][:, 2][event.ind] == min_z]
            else:
                candidates = event.ind
            print("Picking at random in", candidates)
            idxi = numpy.random.choice(candidates)

            scs[axi]._facecolors[:] = (1, 0, 0, 0.5)
            scs[axi]._facecolors[idxi] = (0, 1, 0, 0.5)
            # for i in range(len(options[axi])):
            #     scs[axi]._facecolors[i, :] = (1, 0, 0, 0.5)
            # scs[axi]._facecolors[idxi, :] = (0, 1, 0, 0.5)

            valids = numpy.all(params[axj][:, shared] == params[axi][idxi, shared], axis=1)
            valids_idx = numpy.where(valids)[0]
            if len(valids_idx) == 1:
                idxj = valids_idx[0]

            scs[axj]._facecolors[:] = (1, 0, 0, 0.05)
            scs[axj]._facecolors[valids_idx] = (0, 1, 0, 1)
            # for i, valid in enumerate(valids):
            #     if valid:
            #         scs[axj]._facecolors[i, :] = (0, 1, 0, 0.5)
            #     else:
            #         scs[axj]._facecolors[i, :] = (1, 0, 0, 0.5)
            fig.canvas.draw()
        else:
            # handle the situation where several points overlap
            candidates = [c for c in event.ind if valids[c]]
            print("Picking at random in", candidates)
            idxj = numpy.random.choice(candidates)

            scs[axj]._facecolors[:] = (1, 0, 0, 0.05)
            scs[axj]._facecolors[idxj] = (0, 1, 0, 1)
            # for i in range(len(options[axj])):
            #     scs[axj]._facecolors[i,:] = (1, 0, 0, 0.5)
            # scs[axj]._facecolors[idxj,:] = (0, 1, 0, 0.5)
            fig.canvas.draw()

    fig.canvas.mpl_connect("pick_event", onpick)
    pyplot.show(block=True)
    assert idxi is not None and idxj is not None, "User did not pick any point!"
    if axi == 0:
        return idxi, idxj
    else:
        return idxj, idxi


def give_score(img, label):
    fig = pyplot.figure()
    # fig.canvas.set_window_title("Score image")

    pyplot.subplots_adjust(left=0.25, bottom=0.25)

    # pyplot.imshow(img, interpolation=None, vmin=0, vmax=int(0.9*numpy.max(img)))
    pyplot.imshow(img, interpolation=None, vmax=int(0.9*numpy.max(img)))
    pyplot.colorbar()
    pyplot.grid(True)

    axslider = pyplot.axes([0.25, 0.1, 0.65, 0.03])
    slider = widgets.Slider(axslider, label, 0, 100, valinit=50)

    pyplot.show(block=True)
    return slider.val / 100


if __name__ == "__main__":
    import numpy
    import objectives

    objs = [objectives.Bleach(), objectives.ScoreInv("Quality")]

    grid = numpy.meshgrid(*[numpy.linspace(0, 1, 5) for i in range(3)])
    params = numpy.vstack(map(numpy.ravel, grid)).T

    options1 = numpy.random.random((len(params), 2))
    options2 = numpy.random.random((len(params), 2))

    select_double(params, params, options1, options2, objs)




from setuptools import setup, find_packages

setup(
    name='abberior-sted',
    version='0.0.1',
    description="Interface to Imspector control software for Abberior STED microscopes",
    long_description=open("README.md").read(),
    long_description_content_type='text/markdown',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    requires_python=">=3.8",
    keywords="abberior sted imspector specpy",
    license="CC-BY-4.0 license",
    install_requires=[
        "scikit-image",
        "numpy",
        "scipy",
        "matplotlib",
        "pyyaml"
    ],  # And any other dependencies foo needs
    packages=["abberior"],
    include_package_data=True,
)
# I do not require specpy to be installed
